@@ -30,13 +30,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * @(#) $Header: /tcpdump/master/tcpdump/ipproto.h,v 1.1.2.1 2003/11/24 20:31:22 guy Exp $ (LBL)
+ * @(#) $Header: /tcpdump/master/tcpdump/ipproto.h,v 1.4.2.1 2005/05/20 21:15:46 hannes Exp $ (LBL)
  *
  * From:
  *	@(#)in.h	8.3 (Berkeley) 1/3/94
  * $FreeBSD: src/sys/netinet/in.h,v 1.38.2.3 1999/08/29 16:29:34 peter Exp $
  */
 
+extern struct tok ipproto_values[];
+
 #ifndef IPPROTO_IP
 #define	IPPROTO_IP		0		/* dummy for IP */
 #endif
@@ -58,8 +60,8 @@
 #ifndef IPPROTO_EGP
 #define	IPPROTO_EGP		8		/* exterior gateway protocol */
 #endif
-#ifndef IPPROTO_IGRP
-#define IPPROTO_IGRP		9
+#ifndef IPPROTO_PIGP
+#define IPPROTO_PIGP		9
 #endif
 #ifndef IPPROTO_UDP
 #define	IPPROTO_UDP		17		/* user datagram protocol */
@@ -113,8 +115,8 @@
 #ifndef IPPROTO_ND
 #define	IPPROTO_ND		77		/* Sun net disk proto (temp.) */
 #endif
-#ifndef IPPROTO_IGRP
-#define	IPPROTO_IGRP		88		/* Cisco/GXS IGRP */
+#ifndef IPPROTO_EIGRP
+#define	IPPROTO_EIGRP		88		/* Cisco/GXS IGRP */
 #endif
 #ifndef IPPROTO_OSPF
 #define IPPROTO_OSPF		89
@@ -128,6 +130,9 @@
 #ifndef IPPROTO_VRRP
 #define IPPROTO_VRRP		112
 #endif
+#ifndef IPPROTO_PGM
+#define IPPROTO_PGM             113
+#endif
 #ifndef IPPROTO_SCTP
 #define IPPROTO_SCTP		132
 #endif
