@@ -1,4 +1,4 @@
-/* @(#) $Header: /tcpdump/master/tcpdump/ospf.h,v 1.13 2004/01/27 13:33:24 hannes Exp $ (LBL) */
+/* @(#) $Header: /tcpdump/master/tcpdump/ospf.h,v 1.16 2004/09/20 14:56:34 hannes Exp $ (LBL) */
 /*
  * Copyright (c) 1991, 1993, 1994, 1995, 1996, 1997
  *	The Regents of the University of California.  All rights reserved.
@@ -43,6 +43,7 @@
 #define	OSPF_OPTION_EA	0x10	/* EA bit: External Attribute capable */
 #define	OSPF_OPTION_DC	0x20	/* DC bit: Demand circuit capable */
 #define	OSPF_OPTION_O	0x40	/* O bit: Opaque LSA capable */
+#define	OSPF_OPTION_DN	0x80	/* DN bit: Up/Down Bit capable - draft-ietf-ospf-2547-dnbit-04 */
 
 /* ospf_authtype	*/
 #define	OSPF_AUTH_NONE		0	/* No auth-data */
@@ -63,13 +64,14 @@
 #define	LS_TYPE_ASE		5   /* ASE  */
 #define	LS_TYPE_GROUP		6   /* Group membership (multicast */
 				    /* extensions 23 July 1991) */
-#define	LS_TYPE_NSSA            7   /* rfc1587 - Not so Stubby Areas */
+#define	LS_TYPE_NSSA            7   /* rfc3101 - Not so Stubby Areas */
 #define	LS_TYPE_OPAQUE_LL       9   /* rfc2370 - Opaque Link Local */
 #define	LS_TYPE_OPAQUE_AL      10   /* rfc2370 - Opaque Link Local */
 #define	LS_TYPE_OPAQUE_DW      11   /* rfc2370 - Opaque Domain Wide */
 
 #define LS_OPAQUE_TYPE_TE       1   /* rfc3630 */
 #define LS_OPAQUE_TYPE_GRACE    3   /* rfc3623 */
+#define LS_OPAQUE_TYPE_RI       4   /* draft-ietf-ospf-cap-03 */
 
 #define LS_OPAQUE_TE_TLV_ROUTER 1   /* rfc3630 */
 #define LS_OPAQUE_TE_TLV_LINK   2   /* rfc3630 */
@@ -101,6 +103,8 @@
 #define LS_OPAQUE_GRACE_TLV_REASON_SW_UPGRADE  2 /* rfc3623 */
 #define LS_OPAQUE_GRACE_TLV_REASON_CP_SWITCH   3 /* rfc3623 */
 
+#define LS_OPAQUE_RI_TLV_CAP             1 /* draft-ietf-ospf-cap-03 */
+
 /*************************************************
  *
  * is the above a bug in the documentation?
@@ -208,13 +212,20 @@ struct lsa {
 	    u_int8_t data[1]; /* may repeat   */
 	} un_te_lsa_tlv;
 
-        /* Grace LSA */
+        /* Opaque Grace LSA */
         struct {
 	    u_int16_t type;
 	    u_int16_t length;
 	    u_int8_t data[1]; /* may repeat   */
 	} un_grace_tlv;
 
+        /* Opaque Router information LSA */
+        struct {
+	    u_int16_t type;
+	    u_int16_t length;
+	    u_int8_t data[1]; /* may repeat   */
+	} un_ri_tlv;
+
         /* Unknown LSA */
         struct unknown {
 	    u_int8_t data[1]; /* may repeat   */
