@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-eap.c,v 1.1 2004/03/25 03:29:53 mcr Exp $";
+    "@(#) $Header: /tcpdump/master/tcpdump/print-eap.c,v 1.3 2004/04/23 19:03:39 mcr Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -59,13 +59,13 @@ eap_print(netdissect_options *ndo,
 	eap = (const struct eap_packet_t *)cp;
 	ND_TCHECK(eap->data);
 
-        ND_PRINT("EAP code=%u id=%u length=%u ", 
-		 eap->code, eap->id, (eap->length[0]<<8) + eap->length[1]);
+        ND_PRINT((ndo, "EAP code=%u id=%u length=%u ", 
+		  eap->code, eap->id, (eap->length[0]<<8) + eap->length[1]));
 
         if (!ndo->ndo_vflag)
             return;
 
 trunc:
-
+	;
 }
 
