@@ -26,7 +26,7 @@
 
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-isoclns.c,v 1.133.2.12 2005/06/16 01:14:52 guy Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/tcpdump/print-isoclns.c,v 1.133.2.15 2005/07/11 20:24:33 hannes Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -780,6 +780,7 @@ static int clnp_print (const u_int8_t *pptr, u_int length)
 
         if (clnp_flags & CLNP_SEGMENT_PART) {
             	clnp_segment_header = (const struct clnp_segment_header_t *) pptr;
+                TCHECK(*clnp_segment_header);
                 printf("\n\tData Unit ID: 0x%04x, Segment Offset: %u, Total PDU Length: %u",
                        EXTRACT_16BITS(clnp_segment_header->data_unit_id),
                        EXTRACT_16BITS(clnp_segment_header->segment_offset),
@@ -840,7 +841,7 @@ static int clnp_print (const u_int8_t *pptr, u_int length)
                             if (tlen < source_address_length+1) {
                                     printf("\n\t    NSAP address goes past end of option");
                                     break;
-                    	    }
+                            }
                             if (source_address_length > 0) {
                                     source_address=(tptr+1);
                                     TCHECK2(*source_address, source_address_length);
@@ -1398,9 +1399,9 @@ trunctlv:
  */
 
 static int
-isis_print_is_reach_subtlv (const u_int8_t *tptr,int subt,int subl,const char *ident) {
+isis_print_is_reach_subtlv (const u_int8_t *tptr,u_int subt,u_int subl,const char *ident) {
 
-        int priority_level,bandwidth_constraint;
+        u_int priority_level,bandwidth_constraint;
         union { /* int to float conversion buffer for several subTLVs */
             float f; 
             u_int32_t i;
@@ -1430,7 +1431,7 @@ isis_print_is_reach_subtlv (const u_int8_t *tptr,int subt,int subl,const char *i
 	    break;
         case ISIS_SUBTLV_EXT_IS_REACH_IPV4_INTF_ADDR:
         case ISIS_SUBTLV_EXT_IS_REACH_IPV4_NEIGHBOR_ADDR:
-            if (subl >= 4)
+            if (subl >= sizeof(struct in_addr))
               printf(", %s", ipaddr_string(tptr));
             break;
         case ISIS_SUBTLV_EXT_IS_REACH_MAX_LINK_BW :
@@ -1604,7 +1605,7 @@ static int
 isis_print_extd_ip_reach (const u_int8_t *tptr, const char *ident, u_int16_t afi) {
 
     char ident_buffer[20];
-    u_int8_t prefix[16]; /* shared copy buffer for IPv4 and IPv6 prefixes */
+    u_int8_t prefix[sizeof(struct in6_addr)]; /* shared copy buffer for IPv4 and IPv6 prefixes */
     u_int metric, status_byte, bit_length, byte_length, sublen, processed, subtlvtype, subtlvlen;
 
     if (!TTEST2(*tptr, 4))
@@ -1634,7 +1635,7 @@ isis_print_extd_ip_reach (const u_int8_t *tptr, const char *ident, u_int16_t afi
    
     if (!TTEST2(*tptr, byte_length))
         return (0);
-    memset(prefix, 0, 16);              /* clear the copy buffer */
+    memset(prefix, 0, sizeof(struct in6_addr));              /* clear the copy buffer */
     memcpy(prefix,tptr,byte_length);    /* copy as much as is stored in the TLV */
     tptr+=byte_length;
     processed+=byte_length;
@@ -2262,15 +2263,15 @@ static int isis_print (const u_int8_t *p, u_int length)
 	    break;
 
 	case ISIS_TLV_IP6ADDR:
-	    while (tmp>=16) {
-		if (!TTEST2(*tptr, 16))
+	    while (tmp>=sizeof(struct in6_addr)) {
+		if (!TTEST2(*tptr, sizeof(struct in6_addr)))
 		    goto trunctlv;
 
                 printf("\n\t      IPv6 interface address: %s",
 		       ip6addr_string(tptr));
 
-		tptr += 16;
-		tmp -= 16;
+		tptr += sizeof(struct in6_addr);
+		tmp -= sizeof(struct in6_addr);
 	    }
 	    break;
 #endif
@@ -2360,18 +2361,18 @@ static int isis_print (const u_int8_t *p, u_int length)
 	    break;
 
 	case ISIS_TLV_TE_ROUTER_ID:
-	    if (!TTEST2(*pptr, 4))
+	    if (!TTEST2(*pptr, sizeof(struct in_addr)))
 		goto trunctlv;
 	    printf("\n\t      Traffic Engineering Router ID: %s", ipaddr_string(pptr));
 	    break;
 
 	case ISIS_TLV_IPADDR:
-	    while (tmp>=4) {
-		if (!TTEST2(*tptr, 4))
+	    while (tmp>=sizeof(struct in_addr)) {
+		if (!TTEST2(*tptr, sizeof(struct in_addr)))
 		    goto trunctlv;
 		printf("\n\t      IPv4 interface address: %s", ipaddr_string(tptr));
-		tptr += 4;
-		tmp -= 4;
+		tptr += sizeof(struct in_addr);
+		tmp -= sizeof(struct in_addr);
 	    }
 	    break;
 
@@ -2401,21 +2402,21 @@ static int isis_print (const u_int8_t *p, u_int length)
 	    printf(", Flags: [%s]", ISIS_MASK_TLV_SHARED_RISK_GROUP(*tptr++) ? "numbered" : "unnumbered");
 	    tmp--;
 
-	    if (tmp < 4)
+	    if (tmp < sizeof(struct in_addr))
 	        break;
-	    if (!TTEST2(*tptr,4))
+	    if (!TTEST2(*tptr,sizeof(struct in_addr)))
                 goto trunctlv;
 	    printf("\n\t      IPv4 interface address: %s", ipaddr_string(tptr));
-	    tptr+=4;
-	    tmp-=4;
+	    tptr+=sizeof(struct in_addr);
+	    tmp-=sizeof(struct in_addr);
 
-	    if (tmp < 4)
+	    if (tmp < sizeof(struct in_addr))
 	        break;
-	    if (!TTEST2(*tptr,4))
+	    if (!TTEST2(*tptr,sizeof(struct in_addr)))
                 goto trunctlv;
 	    printf("\n\t      IPv4 neighbor address: %s", ipaddr_string(tptr));
-	    tptr+=4;
-	    tmp-=4;
+	    tptr+=sizeof(struct in_addr);
+	    tmp-=sizeof(struct in_addr);
 
 	    while (tmp>=4) {
                 if (!TTEST2(*tptr, 4))
